package routing

import (
	"context"

	stdlib "orus.io/xbus/xbus-stdlib/lib"
	"orus.io/xbus/xbus-stdlib/service"

	libjq_go "github.com/flant/libjq-go"
	"github.com/flant/libjq-go/pkg/jq"

	"xbus.io/go-xbus/v3"
	"xbus.io/go-xbus/v3/api"
)

// NewRouter initialize a Router
func NewRouter(actor *api.Actor, settings service.Settings) (*Router, error) {
	return &Router{
		settings: settings,
	}, nil
}

// NewRouterAsWorker returns a Worker
func NewRouterAsWorker(actor *api.Actor, settings service.Settings) (service.Worker, error) {
	return NewRouter(actor, settings)
}

// Router is a Worker that updates the routing table
type Router struct {
	settings service.Settings

	extractKeyJq *jq.JqProgram
	table        TableStore
}

// Startup setup the routing table and compiles the jq programs
func (r *Router) Startup() error {
	table, err := NewTableStore("store", r.settings)
	if err != nil {
		return err
	}
	extractKeyJq, err := r.settings.String("extract-key-jq")
	if err != nil {
		return err
	}
	extractKeyJqProg, err := libjq_go.Jq().Program(extractKeyJq).Precompile()
	if err != nil {
		return err
	}
	r.extractKeyJq = extractKeyJqProg
	r.table = table
	return nil
}

// Shutdown closes the routing table
func (r *Router) Shutdown() error {
	return r.table.Close()
}

// Process ...
func (r *Router) Process(
	ctx context.Context, req api.ActorProcessRequest,
	inputs service.Inputs, outputs service.Outputs, log xbus.ZeroLogger,
) error {
	env, err := inputs.Get(api.DefaultInput)
	if err != nil {
		return err
	}

	_, body, err := stdlib.UniqueStringMessageContext(ctx, env)
	if err != nil {
		return err
	}

	key, err := r.extractKeyJq.RunRaw(body)
	if err != nil {
		return err
	}

	output, err := r.table.Get(key)
	if err != nil {
		return err
	}

	return outputs.Forward(output, env.ID())
}

func init() {
	service.RegisterWorker("routing.table-router", NewRouterAsWorker)
}
